function [Df]=int_DF_Floquet_blow_up(x,tx)

%%% inputs: all intputs are assumed to be of class intval
%%% 1) x = (lambda,a), where
%%%    - lambda is a Floquet multiplier
%%%    - a = (a_1,a_2,a_3,a_4) where a_j in C^(2m-1) is the 
%%%           vector of Fourier coefficients of v_j (j=1,2,3,4)
%%%      Note that x in \C \times C^(4(2m-1)) = C^(8m-3) 
%%% 2) tx = (tL,ta), where
%%%    - tL is the frequency of the periodic orbit
%%%    - ta is the vector of Fourier coefficients of gamma
%%%      Note that x in \R \times C^(2m-1)

m=(length(x)+3)/8;

%%% Data from the periodic orbit

tL = tx(1);
ta = tx(2:end);

lambda = x(1); %%% A Floquet multiplier
a1 = x(2:2*m);
a2 = x(2*m+1:4*m-1);
a3 = x(4*m:6*m-2);
a4 = x(6*m-1:8*m-3);

Df=intval(zeros(8*m-3));

%%%%%%%%%%%%%%%%%%%%
%%% First column %%%
%%%%%%%%%%%%%%%%%%%%

D_lambda_f1=a1;
D_lambda_f2=a2;
D_lambda_f3=a3;
D_lambda_f4=a4;

Df(2:end,1)=[D_lambda_f1;D_lambda_f2;D_lambda_f3;D_lambda_f4];

iIm=intval(eye(2*m-1));

%%%%%%%%%%%%%%%%%
%%% First row %%%
%%%%%%%%%%%%%%%%%

k0=1;
Df(1,m+1-k0:m+1+k0)=2*sum(a1(m-k0:m+k0));
Df(1,3*m-k0:3*m+k0)=2*sum(a2(m-k0:m+k0));
Df(1,5*m-1-k0:5*m-1+k0)=2*sum(a3(m-k0:m+k0));
Df(1,7*m-2-k0:7*m-2+k0)=2*sum(a4(m-k0:m+k0));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% We define the rest blockwise %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

k=intval(-m+1:m-1)';

%%% Da1f1 %%%
Df(2:2*m,2:2*m)=diag(1i*tL*k+lambda);

%%% Da2f1 %%%
Df(2:2*m,2*m+1:4*m-1)=-iIm;

%%% Da2f2 %%%
Df(2*m+1:4*m-1,2*m+1:4*m-1)=diag(1i*tL*k+lambda);

%%% Da3f2 %%%
Df(2*m+1:4*m-1,4*m:6*m-2)=-iIm;

%%% Da3f3 %%%
Df(4*m:6*m-2,4*m:6*m-2)=diag(1i*tL*k+lambda);

%%% Da4f3 %%%
Df(4*m:6*m-2,6*m-1:8*m-3)=-iIm;

%%% Da1f4 %%%
ta2=quadratic_sumFFTcomplex(ta);
conv = intval(zeros(2*m-1));
for k1=-m+1:m-1
    for ell=-m+1:m-1
        conv(k1+m,ell+m)=ta2(k1-ell+2*m-1);
    end
end
Df(6*m-1:8*m-3,2:2*m)=120*iIm+3*conv;

%%% Da2f4 %%%
Df(6*m-1:8*m-3,2*m+1:4*m-1)=154*iIm;

%%% Da3f4 %%%
Df(6*m-1:8*m-3,4*m:6*m-2)=71*iIm;

%%% Da4f4 %%%
Df(6*m-1:8*m-3,6*m-1:8*m-3)=diag(1i*tL*k+lambda)+14*iIm;

end

function [s]=quadratic_sumFFTcomplex(a)

m=(length(a)+1)/2;

m1=(2^ceil(log(4*m-1)/log(2))-4*m)/2;   % Hence 4*m+2*m1 is a power of 2 %

ta=[intval(zeros(m+m1+1,1));a;intval(zeros(m+m1,1))];
tu=verifyfft(ifftshift(ta),-1);

F=fftshift(verifyfft(tu.*tu,1));

s=(4*m+2*m1)*F(m1+3:4*m+m1-1);

end