function [f]=int_F_Floquet_blow_up(x,tx)

%%% inputs: all intputs are assumed to be of class intval
%%% 1) x = (lambda,a), where
%%%    - lambda is a Floquet multiplier
%%%    - a = (a_1,a_2,a_3,a_4) where a_j in C^(2m-1) is the 
%%%           vector of Fourier coefficients of v_j (j=1,2,3,4)
%%%      Note that x in \C \times C^(4(2m-1)) = C^(8m-3) 
%%% 2) tx = (tL,ta), where
%%%    - tL is the frequency of the periodic orbit
%%%    - ta is the vector of Fourier coefficients of gamma
%%%      Note that x in \R \times C^(2m-1)

m=(length(x)+3)/8;

%%% Data from the periodic orbit

tL = tx(1);
ta = tx(2:end);

lambda = x(1); %%% Floquet multiplier
a1 = x(2:2*m);
a2 = x(2*m+1:4*m-1);
a3 = x(4*m:6*m-2);
a4 = x(6*m-1:8*m-3);

k0=1;
f0=sum(a1(m-k0:m+k0))^2+sum(a2(m-k0:m+k0))^2+sum(a3(m-k0:m+k0))^2+sum(a4(m-k0:m+k0))^2-1;

k=intval(-m+1:m-1)';
mu=1i*tL*k;
[ta_ta_a1]=int_cubic_sumFFTcomplex(ta,ta,a1);

f1=mu.*a1+lambda*a1-a2;
f2=mu.*a2+lambda*a2-a3;
f3=mu.*a3+lambda*a3-a4;
f4=mu.*a4+lambda*a4+120*a1+154*a2+71*a3+14*a4+3*ta_ta_a1;

f=[f0;f1;f2;f3;f4];

end