function [x]=padding_Floquet(x,n)

%%% inputs: (1) x = (lambda,a), where
%%%    - lambda is a Floquet multiplier
%%%    - a = (a_1,a_2,a_3,a_4) where a_j in C^(2m-1) is the 
%%%           vector of Fourier coefficients of v_j (j=1,2,3,4)
%%%      Note that x in \C \times C^(4(2m-1)) = C^(8m-3) 
%%%         (2) n : the extra 0's

m=(length(x)+3)/8;

lambda = x(1); %%% A Floquet multiplier
a1 = x(2:2*m);
a2 = x(2*m+1:4*m-1);
a3 = x(4*m:6*m-2);
a4 = x(6*m-1:8*m-3);

a1_ext=padding(a1,n);
a2_ext=padding(a2,n);
a3_ext=padding(a3,n);
a4_ext=padding(a4,n);
x=[lambda;a1_ext;a2_ext;a3_ext;a4_ext];

end

function [a]=padding(a,n)

%%% inputs: (1) a is the vector of complex Fourier coefficients
%%%         (2) n: the extra 0

if n>0   
    a = [zeros(n,1);a;zeros(n,1)];
else
    a(1:-n)=[];
    a(end+1+n:end)=[];
end

end