
%%%% script for the computation and validation of the manifold for the
%%%% bridge equation as in paper
%% R.C, J.P. Lessard, J.D Mireles Parameterization of invariant manifolds for periodic
%  orbits (II): a-posteriori analysis and computer assisted error bounds


%%%%% Compute peridic orbit %%%%%%
cd Periodic_Orbit
script_proof_gamma
cd ..

r_gamma=r_minus;   
tau=2*pi/tx(1);
gamma_k=tx(2:end);
mgamma=(length(gamma_k)+1)/2;

%%%% Computation of the real  Floquet Normal form %%%%%%%%

Ak_tau_per=compute_Ak_bridge(gamma_k);     % compute Four. coefficients of Jacobian
Ak=Ak_tau_per;

%%% Newton scheme for Floquet system
%%% R is the exponent matrix
%%% Q=Q(t) is the periodic matrix function

disp('Newton scheme for the Floquet system')

[R,Q,Jac_num]=newton_floquet_bridge(gamma_k,Ak,mgamma,tau);       
                                        
if isempty(R)==1
    disp(' Newton scheme does not converge');
    return
end
disp(' Success !')

%%%%% Move to 2T periodic 
S=size(Ak_tau_per);
Ak=zeros(S(1),S(2),(S(3)-1)*2+1);
Ak(:,:,1:2:end)=Ak_tau_per; 

S=size(Q);
Qk=zeros(S(1),S(2),(S(3)-1)*2+1);
Qk(:,:,1:2:end)=Q; 
Q=Qk;
[R,Q,Jac_num]=newton_floquet_bridge2T(Q,R,Ak,tau);


m=(size(Q,3)+1)/2;


% compute numerical inverse of Q
disp('compute Q_inverse');
time=linspace(0,2*tau,501);
time(end)=[];
Q_inv_t=zeros([size(Q(:,:,1)),length(time)]);
mQ=(size(Q,3)+1)/2;

for ll=1:length(time)
    t=time(ll);
    Q_t=zeros(size(Q(:,:,1)));
    for k=-(mQ-1):(mQ-1)
        Q_t=Q_t+Q(:,:,mQ+k)*exp(1i*2*pi/(2*tau)*k*t);
    end
    Q_inv_t(:,:,ll)=inv(Q_t);
end
Qinv=fft(real(Q_inv_t),[],3)/length(time);
Qinv=cat(3,conj(Qinv(:,:,mQ:-1:2)),Qinv(:,:,1:mQ));
%%%%%%%%%%%%
% define P=(Qinv)^T
% define B=-A^T
% define W=-R^T
P=zeros(size(Qinv));
Bk=zeros(size(Ak));
for i=1:size(P,3)
    P(:,:,i)=Qinv(:,:,i).';
end
for i=1:size(Bk,3)
    Bk(:,:,i)=-Ak(:,:,i).';
end
W=-R.';

% Newton scheme for P and W     
[W,P,Jac_num]=newton_floquet_bridge2T(P,W,Bk,tau); 

%% Reconstruct Qinv as P^T
Qinv=zeros(size(P));
for i=1:size(P,3)
    Qinv(:,:,i)=P(:,:,i).';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% High order parameterization   %%%%%%%%%%%%%%%%%
%%%%%%         in the 2T-periodic space         %%%%%%%%%%%%

% move gamma to a 2T periodic function
gamma_k=gamma_k.';

S=size(gamma_k);
gamma2T=zeros(S(1),(S(2)-1)*2+1);
gamma2T(:,1:2:end)=gamma_k;


disp('Computing the Parametrization');

scaleEigs=[5,20];
dim_man=2;
m_manifold=60;
m_manifold=max(m_manifold,2*mgamma);
N_tilde=3;
N=380;
D2=[0,0,0,0;0,0,0,0;0,0,0,0;-1,1,1,1]; 
stability='S';

rfloquet=0;
[alphas,lambda]=high_order_coefficients_bridge(gamma2T,2*tau,Q,R,m_manifold,N_tilde,stability,D2,dim_man,scaleEigs,Qinv,rfloquet);
[f,f_int,Jac, Jac_int, alphas]=compute_fN_manifold_bridge(alphas,lambda, tau);


der_i=alphas(1,1).fc(1,:);
 for i=1:3
     der_i=1i*pi/tau*[-m_manifold+1:m_manifold-1].*der_i;
     alphas(1,1).fc(i+1,:)=der_i;
 end

nu_A=1.0005;
%r_gamma=7.595549832526767e-13;

Epsilon(1,1).n=[0,0];
Epsilon(1,1).r=r_gamma*ones(4,1);

Epsilon(2,1).n=[1,0];
Epsilon(1,2).n=[0,1];
if lambda(1)==-7
    Epsilon(2,1).r=scaleEigs(1)*4.4942e-12*ones(4,1);
    Epsilon(1,2).r=scaleEigs(2)*4.9807e-13*ones(4,1);
else
    Epsilon(1,2).r=scaleEigs(1)*4.4942e-12*ones(4,1);
    Epsilon(2,1).r=scaleEigs(2)*4.9807e-13*ones(4,1);
end
for i=1:numel(alphas)
    alphas(i).eps=zeros(4,1);
end
alphas(1,1).eps=Epsilon(1,1).r;
alphas(2,1).eps=Epsilon(2,1).r;
alphas(1,2).eps=Epsilon(1,2).r;

Lambda_int=midrad(lambda,[4.4942e-12,4.9807e-13]);
space_dim=4;

par2=[space_dim, dim_man,N_tilde];

[rad_param, aquad,aquad0_star,alphas]=proof_high_order_terms_bridgeNM(f_int, Jac_int,Lambda_int,tau,par2,nu_A,alphas,Epsilon,m_manifold);
% alpha.norm is only the nu norm of the first component
for i=1:numel(alphas)
   if sum(alphas(i).n)>1
       alphas(i).eps=rad_param*ones(4,1);
   end
end

%%%%%% alpha from N to tilde N %%%%%

N_star=3*N_tilde+6;
M_manifold=80;
for i=1:numel(alphas)
    if isempty(alphas(i).fc)==0
        m_old=(size(alphas(i).fc,2)+1)/2;
        alphas(i).fc=[zeros(4,M_manifold-m_old),alphas(i).fc,zeros(4,M_manifold-m_old)];
    end
end
[J,A,B,A_bound]=compute_gradient2_int(alphas,lambda,tau,N_tilde+1,N_star,aquad,intval(nu_A));

[rad_extra,aquad,alphas]=proof_super_term(A,B,intval(nu_A),tau,N_tilde,N_star,alphas,lambda,A_bound,N)

%%%%%% COMPUTE VALIDATION VALUES %%%%%

Gamma=gamma2T;
m=(size(Gamma,2)+1)/2;
ipi=intval('pi');
i6=intval(6);
i4=intval(4);
i3=intval(3);
i2=intval(2);
i1=intval(1);
itau=midrad(tau,r_gamma);
iT=i2*itau;
T=mid(iT);


tol=10^-11;
nu=nu_A;
%R_alpha=T/(2*pi)*log(nu_A);
r=inf(iT/(intval(2)*intval(pi))*log(intval(nu)));
ir=intval(r);
ilambda=Lambda_int;%midrad(lambda,r_v);

NU=1;


%%%% compute A= norm (Df)_r^\infty

Df_gamma=intval([0,1,0,0;0,0,1,0;0,0,0,1;120,154,71,14]);
gamma2=aquad(1,1).fc;
mid_value=(size(gamma2,2)+1)/2;
gamma2=gamma2(mid_value+[-(2*m-2):(2*m-2)]);




Df_gamma(4,1)=Df_gamma(4,1)+i3*sum((exp(i2*ipi*ir/iT).^(intval(abs(-2*m+2:2*m-2)))).*abs(gamma2))+i3*aquad(1,1).eps(1);
A=norm(Df_gamma.sup,inf);

%%%% r-norm of a_alpha^1, for any alpha with bound %%%%%
m_man=(size(alphas(1,2).fc,2)+1)/2;
cont=1;
VEC1_al=[];
for j=1:numel(alphas)
    if isempty(alphas(j).n)==0
       VEC1_al(cont,:)=alphas(j).n;
       alphanorm(:,cont)=intval(abs(alphas(j).fc))*(exp(i2*ipi*ir/iT).^(intval(abs(-m_man+1:m_man-1)))).'+alphas(j).eps;
       cont=cont+1;
    end
end

SUM_alphanorm=sum(alphanorm(1,:),2); % sum of alphas up to Nstar
        
%%%% compute A_alpha    

Aal=[];
VEC_al=[];
cont=1;


% |alpha|<2*Nstar


  for n=2:numel(aquad)
      [i,j]=ind2sub(size(aquad),n);
      if i+j-2<=2*N_star
   
        VEC_al(cont,:)=[i-1,j-1];
        if isempty(aquad(n).fc)==0
            s=(size(aquad(n).fc,2)+1)/2;
            Aal(cont)=sup((exp(i2*ipi*ir/iT).^(intval(abs(-s+1:s-1)))*(abs(aquad(n).fc)).')+aquad(n).eps);
        else
            Aal(cont)=sup(intval(aquad(n).norm)+intval(aquad(n).eps));
        end
        cont=cont+1;
      end
      
  end
   
  
  %%%%%   2*Nstar<|alpha|<=N
  for modN=2*N_star+1:2*N
     al1=modN:-1:0;
     al2=modN-al1; 
     AL=[al1.',al2.'];
     VEC_al=cat(1,VEC_al,AL);
     Aal=[Aal,(prod(AL+1,2)*rad_extra^2).'+2*rad_extra*SUM_alphanorm];
  end
         
 
A2=Aal;
Aal=3*Aal.';


Ctilde=exp(sum(Aal.*(NU.^(sum(VEC_al,2)))./abs((VEC_al*lambda.'))));
ErrorBound=aposteriori_error_bound_bridge(N,N_star,alphas,A2,VEC_al,NU,rad_extra,alphanorm);

a_al_inf_r=alphanorm(4,:);
for modN=N_star+1:N
     al1=modN:-1:0;
     al2=modN-al1; 
     AL=[al1.',al2.'];
     VEC1_al=cat(1,VEC1_al,AL);
     a_al_inf_r=[a_al_inf_r, rad_extra*ones(1,size(AL,1))];
end



rho1=max(sup(a_al_inf_r(:,2:end)*NU.^(sum(VEC1_al(2:end,:),2))));
rho=rho1+0.1*rho1;


mu_star=min(abs(real(lambda)));
M1=i1;
max_gamma=sum(abs(Gamma(1,:)));
M2=i6*(max_gamma+rho);

%%%%%    A-posteriori validation %%%%
val1=0;
val2=0;
if (N+1)>A/mu_star
    val1=1;
else
    'error1'
end

delta=sup((i2*Ctilde)*intval(ErrorBound)/((intval(N+1))*intval(mu_star)-intval(A)));

if  delta<min(inf(exp(-i1)*(intval((N+1))*intval(mu_star)-intval(A))/(i2*i4*ipi*intval(M1)*intval(M2)*Ctilde)),rho-rho1)
      val2=1;
else
    'error2'
end

if val1*val2>0
    disp('The parameterization is rigorously validated')
    display(['r = ' num2str(r)]);
    display(['NU = ' num2str(NU)]);
    display(['r-NU-norm of H = ' num2str(delta)]);
    display(['A-posteriori Error bound = ' num2str(sup(ErrorBound))]);
else
    display('validation fails');
end


%%% consider the 1-dimensional manifold in R^3 ( only the first 3 components
%%%  sigma_2=0 %%%%
alphas_1=alphas(:,1);
for i=1:numel(alphas_1)
    alphas_1(i).fc(4,:)=[];
end
alphas_1=alphas_1.';
plot_parametrization_complex(2*tau,alphas_1,NU);

%%%  sigma_1=0 %%%%

alphas_2=alphas(1,:);
for i=1:numel(alphas_2)
    alphas_2(i).fc(4,:)=[];
end
plot_parametrization_complex(2*tau,alphas_2,NU);

 B=[];
 for i=2:numel(alphas)
     if isempty(alphas(i).n)==0
         if sum(alphas(i).n)>3
             B=[B,alphas(i).eps(1)];
         end
     end
 end