function [DF]=DF_PO_blow_up(x)

%%% input: x = (L,a), where
%%%    - L is the frequency of the periodic solution
%%%    - a is the vector of Fourier coefficients in C^(2m-1)

m=length(x)/2;

L = x(1); %%% The frequency of the periodic orbit
a = x(2:end);

DF=zeros(2*m);

%%% D_{x}F (First row) %%%
DF(1,m-2:m+4)=1; %%% Since F0 = sum_{|k|<=3} a_k

%%% D_{L}F1 (Part of the first column) %%%
k=(-m+1:m-1)';
dmu_dL=4*L^3*k.^4-42*1i*L^2*k.^3-142*L*k.^2+154*1i*k;
DF(2:end,1)=dmu_dL.*a;

%%% D_{a}F1 (the rest) %%%

a2=quadratic_sumFFTcomplex(a);
mu=L^4*k.^4-14*1i*L^3*k.^3-71*L^2*k.^2+154*1i*L*k+120;

dF1_da = zeros(2*m-1);

for k=-m+1:m-1
    for ell=-m+1:m-1
        dF1_da(k+m,ell+m)=a2(k-ell+2*m-1);
    end
end

DF(2:end,2:end)=diag(mu)+3*dF1_da;

end

function [s]=quadratic_sumFFTcomplex(c)

m=(length(c)+1)/2;

ta=[zeros(m,1);c;zeros(m,1)];
tu=ifft(ifftshift(ta));

F=fftshift(fft(tu.*tu));

s=(4*m-1)*F(2:4*m-2);

end
