function [DF]=int_DF_PO_blow_up(x)

%%% input: x = (L,a), where
%%%    - L is the frequency of the periodic solution
%%%    - a is the vector of Fourier coefficients in C^(2m-1)

m=length(x)/2;

L = x(1); %%% The frequency of the periodic orbit
a = x(2:end);

DF=intval(zeros(2*m));

%%% D_{x}F (First row) %%%
DF(1,m-2:m+4)=intval(1); %%% Since F0 = sum_{|k|<=3} a_k

%%% D_{L}F1 (Part of the first column) %%%
k=intval(-m+1:m-1)';
dmu_dL=4*L^3*k.^4-42*1i*L^2*k.^3-142*L*k.^2+154*1i*k;
DF(2:end,1)=dmu_dL.*a;

%%% D_{a}F1 (the rest) %%%

a2=quadratic_sumFFTcomplex(a);
mu=L^4*k.^4-14*1i*L^3*k.^3-71*L^2*k.^2+154*1i*L*k+120;

dF1_da = intval(zeros(2*m-1));

for k=-m+1:m-1
    for ell=-m+1:m-1
        dF1_da(k+m,ell+m)=a2(k-ell+2*m-1);
    end
end

DF(2:end,2:end)=diag(mu)+3*dF1_da;

end

function [s]=quadratic_sumFFTcomplex(a)

m=(length(a)+1)/2;

m1=(2^ceil(log(4*m-1)/log(2))-4*m)/2;   % Hence 4*m+2*m1 is a power of 2 %

ta=[intval(zeros(m+m1+1,1));a;intval(zeros(m+m1,1))];
tu=verifyfft(ifftshift(ta),-1);

F=fftshift(verifyfft(tu.*tu,1));

s=(4*m+2*m1)*F(m1+3:4*m+m1-1);

end
