function [P]=int_rad_poly_gamma(x,nu)

L = intval(x(1)); %%% The frequency of the periodic orbit

m=length(x)/2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Definition of the symmetric operator A %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[DF]=DF_PO_blow_up(x);

A0=inv(DF); %%% A0 is the inverse without imposing the "real" symmetry

[A] = intval(symmetrize(A0));

%%%%%%%%%
%%% Y %%%
%%%%%%%%%

[F]=int_F_PO_blow_up(intval(padding(x,2*m-2)));

F1=F(2:end);
k=intval(-3*m+3:3*m-3)';
mu=L^4*k.^4-14*1i*L^3*k.^3-71*L^2*k.^2+154*1i*L*k+120;

yF=A*padding(F,-(2*m-2));
Y0=abs(yF(1));

y=[F1(1:2*m-2)./mu(1:2*m-2);yF(2:end);F1(4*m-2:6*m-5)./mu(4*m-2:6*m-5)];
Y1=sum(abs(y).*nu.^abs(k));

Y=sup([Y0;Y1]);

%%%%%%%%%%%%%%%%%%%%%%%
%%% Z0, Z1, Z2 & Z3 %%%
%%%%%%%%%%%%%%%%%%%%%%%

[iDF]=int_DF_PO_blow_up(intval(x));

%%% Z0 %%%

nu_power = nu.^abs(intval(-m+1:m-1));

B = abs(intval(eye(2*m))-A*iDF);

Z0=zeros(2,1);

Z0(1) = sup(B(1,1) + intval(max(sup(B(1,2:2*m)./nu_power))));

z011 = sum(B(2:2*m,1).*nu_power');
z012 = intval(max(sup(sum(B(2:2*m,2:2*m).*repmat(nu_power,2*m-1,1)')./nu_power)));
Z0(2) = sup(z011 + z012);

% Before computing the Z bounds, we need some quantities

a=intval(x(2:end));
a_nu_norm=sum(abs(a).*nu_power');

A_a0 = A(1,2:end);
A_a1 = A(2:end,2:end);

A_a1_nu_norm = intval(max(sup(sum(abs(A_a1).*repmat(nu_power,2*m-1,1)')./nu_power)));

k=intval(-m+1:m-1);
tA_a0=(12*k.^4*L^2-84*1i*k.^3*L-142*k.^2).*A_a0;
tB_a0=(4*k.^4*L^3-42*1i*k.^3*L^2-142*k.^2*L+154*1i*k).*A_a0;
tC_a0=(12*k.^4*L-42*1i*k.^3).*A_a0;
tD_a0=(6*k.^4*L-42*1i*k.^3*L-71*k.^2).*A_a0;
tE_a0=(4*k.^4*L-14*1i*k.^3).*A_a0;
tF_a0=(k.^4).*A_a0;

 A_a0_norm = intval(max(sup(abs(A_a0)./nu_power)));
tA_a0_norm = intval(max(sup(abs(tA_a0)./nu_power)));
tB_a0_norm = intval(max(sup(abs(tB_a0)./nu_power)));
tC_a0_norm = intval(max(sup(abs(tC_a0)./nu_power)));
tD_a0_norm = intval(max(sup(abs(tD_a0)./nu_power)));
tE_a0_norm = intval(max(sup(abs(tE_a0)./nu_power)));
tF_a0_norm = intval(max(sup(abs(tF_a0)./nu_power)));

k=(-m+1:m-1)';
nu_power = nu.^abs(k);

tA_a1=12*k.^4*L^2-84*1i*k.^3*L-142*k.^2;
tB_a1=4*k.^4*L^3-42*1i*k.^3*L^2-142*k.^2*L+154*1i*k;
tC_a1=12*k.^4*L-42*1i*k.^3;
tD_a1=6*k.^4*L-42*1i*k.^3*L-71*k.^2;
tE_a1=4*k.^4*L-14*1i*k.^3;
tF_a1=k.^4;

tA_a1_norm = intval(max(sup(abs(tA_a1)./nu_power)));
tB_a1_norm = intval(max(sup(abs(tB_a1)./nu_power)));
tC_a1_norm = intval(max(sup(abs(tC_a1)./nu_power)));
tD_a1_norm = intval(max(sup(abs(tD_a1)./nu_power)));
tE_a1_norm = intval(max(sup(abs(tE_a1)./nu_power)));
tF_a1_norm = intval(max(sup(abs(tF_a1)./nu_power)));

%%% Z1 %%%
Z1=zeros(2,1);

a_ext = intval([zeros(2*m-2,1);abs(a);zeros(2*m-2,1)]);
omega_I = [nu.^-intval(3*m-3:-1:m)';intval(zeros(2*m-1,1));nu.^-intval(m:3*m-3)'];
a2_omega_I=int_cubic_sumFFTcomplex(a_ext,a_ext,omega_I);
z11_temp=abs(a2_omega_I(2*m-1:4*m-3));
mu_m=L^4*m^4-14*1i*L^3*m^3-71*L^2*m^2+154*1i*L*m+120;

Z1(1)=sup(3*abs(A_a0)*z11_temp);
Z1(2)=sup(3*sum((abs(A_a1)*z11_temp).*nu.^intval(-m+1:m-1)')+3*a_nu_norm^2/abs(mu_m));

%%% Z2 %%%

Z2 = zeros(2,1);
Z2(1)=sup(tA_a0_norm*a_nu_norm+6*A_a0_norm*a_nu_norm+2*tB_a0_norm);
Z2(2)=sup(A_a1_nu_norm*(sum(abs(tA_a1.*a).*nu_power)+6*a_nu_norm+2*tB_a1_norm));

%%% Z3 %%%
Z3 = zeros(2,1);
Z3(1)=sup(tC_a0_norm*a_nu_norm+3*A_a0_norm+tA_a0_norm+tD_a0_norm);
Z3(2)=sup(A_a1_nu_norm*(sum(abs(tC_a1.*a).*nu_power)+3+tA_a1_norm+tD_a1_norm));

%%% Z4 %%%
Z4 = zeros(2,1);
Z4(1)=sup(tE_a0_norm+tC_a0_norm+4*tF_a0_norm*a_nu_norm);
Z4(2)=sup(A_a1_nu_norm*(tE_a1_norm+tC_a1_norm+4*sum(abs(tF_a1.*a).*nu_power)));

%%% Z5 %%%
Z5 = zeros(2,1);
Z5(1)=sup(5*tF_a0_norm);
Z5(2)=sup(A_a1_nu_norm*(5*tF_a1_norm));

%%% Radii Polynomials %%%

P = [Z5 Z4 Z3 Z2 Z1+Z0-1 Y];

end