function [R,Q,JJ]=newton_floquet_bridge(gamma,Ak,m,T)


%Newton scheme for the solution of the Floquet system, in unknowns R, C_k
%The initial data for R is given by integration of the variational system
%up to 2*Period and then by computing the matrix logarithm.
%The initial point for C_k is obtained by integrating the Floquet system
%using the above R and then by extracting the Fuorier coefficients 


%global rho sigma beta gamma_0 gamma_k T

options=odeset('AbsTol',1e-16,'RelTol',1e-14);
m=(length(gamma)+1)/2;
    
M=bridge_monodromy2T;                   % first guess for the monodromy matrix
R=logm(real(M))/(T);
R=real(R);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% integration of f'=Jac(gamma(t))*f-f*R, f(0)=I

inif=eye(4);
ini=reshape(inif,[],1);
options=odeset('AbsTol',1e-14,'RelTol',1e-13);
[t,y]=ode45(@system,[0,T],ini,options,gamma);       
[t,y]=ode45(@system,linspace(0,T,size(y,1)),ini,options,gamma);
N=length(t);

for j=1:16                                                                  % fourier coefficients of the solution f(t)
    Comp(:,j)=fft(y(:,j))/N;
end

for j=1:m
    Q(:,:,j)=reshape(Comp(j,:),4,4);
end

clear Comp

%%%%%%%%%% Newton scheme for R and C


Q=cat(3,flip(conj(Q(:,:,2:end)),3),Q);

m=(size(Q,3)+1)/2;
Ak=cat(3,zeros(4,4,2*m),Ak,zeros(4,4,2*m));



NewtonQR
while norm(Q(:,:,end)+Q(:,:,end-1))>10^-16
    Q=cat(3,zeros(4,4,10),Q,zeros(4,4,10));
    Ak=cat(3,zeros(4,4,10),Ak,zeros(4,4,10));
    m=m+10;
    NewtonQR;
    display(['norm(Q(:,:,end)+Q(:,:,end-1)) = ',num2str(   norm(Q(:,:,end)+Q(:,:,end-1)))])
    
  
end
    function NewtonQR
    Ntest=0;
    F=map_floquet(Ak,Q,R,T);
    X=reshape(cat(3,Q(:,:,1:m-1),R,Q(:,:,m:end)),[],1);
    JJ=  jacobian_floquet_bridge_nosym(R,Q,Ak,T);
while norm(F,inf)>9*1e-13  


X1=X-JJ\F;
Q=cat(3,reshape(X1(1:(m-1)*16),4,4,m-1),reshape(X1(16*m+1:end),4,4,m));
R=reshape(X1((m-1)*16+1:m*16),4,4,1);

F=map_floquet(Ak,Q,R,T);
display(['Inf-norm of F(x) = ',num2str( norm(F,inf))])
Ntest=Ntest+1;
    if Ntest>50
        if norm(F,inf)<5*1e-13
            break
        elseif norm(F,inf)>1
            disp('Newton scheme for the Floquet system does not converge');
            disp('Try to change  m');
            R=[];
            Q=[];
            return
        else
            Ntest=Ntest-2;
        end
    end
  X=reshape(cat(3,Q(:,:,1:m-1),R,Q(:,:,m:end)),[],1);   
  JJ=jacobian_floquet_bridge_nosym(R,Q,Ak,T);
end
R=real(R);

end
%%%%%%%%%%%%%%%
    function Mon=bridge_monodromy2T
                        
        X=eye(4);
    
        ini=reshape(X,[],1);
        [t,y]=ode113(@systemMonodromy,[0 T],ini,options,gamma);   % integrate linearized system   
        Mon=reshape(y(end,1:end),4,4);
        
        
        function df=systemMonodromy(t,x,g)
            
         
             gammat=real(sum(exp(1i*((-m+1):1:(m-1))*2*pi*t/T).*(g.')));
            Mon=reshape(x,4,4);            
            J=[0, 1, 0, 0;...
               0, 0, 1, 0;...
               0, 0, 0, 1;...
               -120-3*(gammat^2),-154,-71,-14];
                        

            dfM=J*Mon;
            df=[reshape(dfM,16,1)];
            
        end
        
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function df=system(t,x,g)
        
        f=reshape(x,4,4);
     
          gammat=real(sum(exp(1i*(-m+1:1:m-1)*2*pi*t/T).*(g.')));
       J=[0, 1, 0, 0;...
               0, 0, 1, 0;...
               0, 0, 0, 1;...
               -120-3*(gammat^2),-154,-71,-14];
                        
        dff=J*f-f*R;
        df=[reshape(dff,16,1)];
    end
end
%%%%%%
function F=map_floquet(Ak,Q,R,T)
        m=(size(Q,3)+1)/2;
        dim=size(R,1);
        F=zeros(dim,dim,2*(m-1)+1+1);
        for k=-(m-1):-1
           F(:,:,m+k)=1i*k*2*pi/T*Q(:,:,m+k)+Q(:,:,m+k)*R-convAQ(k,Ak,Q) ;
        end
          F(:,:,m)=sum(Q,3)-eye(dim);
          for k=0:m-1
               F(:,:,m+1+k)=1i*k*2*pi/T*Q(:,:,m+k)+Q(:,:,m+k)*R-convAQ(k,Ak,Q) ;
          end
          
          F=reshape(F,[],1);
end
    function Conv=convAQ(k,A,Q)
        dimA=size(A,1);
        medA=(size(A,3)+1)/2;
        m=(size(Q,3)+1)/2;
    
        l=-(m-1):1:(m-1);
        j=-l+k;
        Conv=reshape(A(:,:,medA+j),dimA,[])*reshape(permute(Q(:,:,m+l),[1,3,2]),[],dimA);
        
        
    end
