
matlabpool('open',2);

%script_proof    % compute periodic orbit
getting_the_orbits
disp(['The periodic orbits has been proved in norm nu=' num2str(nu_gamma)]);
disp(['and radius=' num2str(r)])


par=[sigma,rho,beta];
tau=2*pi/x(1); % period of the orbit  
rgamma=r;

gamma_k=reshape(x(2:end),[],3).';   

gamma=gamma_k(:,m_gamma:end);    % only gamma_k, k>=0
%%%% Computation of the real  Floquet Normal form %%%%%%%%


Ak_tau_per=compute_Ak_Lorenz(gamma,par);     % compute Four. coefficients of Jacobian

S=size(Ak_tau_per);
Ak=zeros(S(1),S(2),(S(3)-1)*2+1);
Ak(:,:,1:2:end)=Ak_tau_per;                  % move to 2T period


%%% Newton scheme for Floquet system
disp('Newton scheme for the Floquet system')

[R,Q,Jac_num]=newton_floquet_lorenz(gamma,Ak,60,par,tau);       
                                        
if isempty(R)==1
    disp(' Newton scheme does not converge');
    return
end
disp(' Success !')
%%%%%%%%%%%%%%%%%%%%

% Proof for the Floquet decomposition
disp('Rigorous enclosure of the Floquet normal form decomposition');

MA=(size(Ak,3)+1)/2;
          
nu=sqrt(nu_gamma);         % nu= decay rate for the orbit norm


m=(size(Q,3)+1)/2;

% compute numerical Jacobian of Floquet system and numercial inverse
Jm=inv(Jac_num);
disp(['Computing enclosing interval for the Floquet system with nu=' num2str(nu)])
tic
[rfloquet,normQ]=radii_pol_floquet_lorenz_analyt_comp(Ak,rgamma,R,Q,tau,Jm,nu);
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Compute iverse of Q(t)
disp(' Computation of inverse of Q(t)')

% compute numerical inverse of Q
time=linspace(0,2*tau,501);
time(end)=[];
Q_inv_t=zeros([size(Q(:,:,1)),length(time)]);
mQ=(size(Q,3)+1)/2;

for ll=1:length(time)
    t=time(ll);
    Q_t=zeros(size(Q(:,:,1)));
    for k=-(mQ-1):(mQ-1)
        Q_t=Q_t+Q(:,:,mQ+k)*exp(1i*2*pi/(2*tau)*k*t);
    end
    Q_inv_t(:,:,ll)=inv(Q_t);
end
Qinv=fft(real(Q_inv_t),[],3)/length(time);
Qinv=cat(3,conj(Qinv(:,:,m:-1:2)),Qinv(:,:,1:m));
%%%%%%%%%%%%
% define P=(Qinv)^T
% define B=-A^T
% define W=-R^T
P=zeros(size(Qinv));
Bk=zeros(size(Ak));
for i=1:size(P,3)
    P(:,:,i)=Qinv(:,:,i).';
end
for i=1:size(Bk,3)
    Bk(:,:,i)=-Ak(:,:,i).';
end

W=-R.';
% Newton for P and W     
[W,P,Jac_num]=newton_floquet_inv_lorenz(P,W,Bk,par,tau);  

%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%
%% Reconstruct Qinv as P^T
Qinv=zeros(size(P));
for i=1:size(P,3)
    Qinv(:,:,i)=P(:,:,i).';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% High order parameterization   %%%%%%%%%%%%%%%%%
%%%%%%         in the 2T-periodic space         %%%%%%%%%%%%

% move gamma to a 2T periodic function
gamma2T=zeros(3,2*(size(gamma,2)-1)+1);
gamma2T(:,1)=gamma(:,1);
gamma2T(:,3:2:size(gamma2T,2))=gamma(:,2:end);
gamma2T=[flip(conj(gamma2T(:,2:end)),2),gamma2T];

%%%%%%%%%%%  compute the parameters function a(theta)

disp('Computing the Parametrization');

dim_man=1;
m_manifold=M;
D2=[0,0,0;-1,1,3;1,1,2]; 


if N<12
    N_manifold=N;
else
    N_manifold=10;
end

[alphas,lambda,v,r_v]=high_order_coefficients(gamma2T,2*tau,Q,R,m_manifold,N_manifold,stability,D2,dim_man,scaleEigs,Qinv,rfloquet);
%%%%%%%%%%%%%%%

disp('solving for the high order terms')


[f,f_int,Jac, Jac_int, alphas,vec_al]=compute_fN_manifold_lorenz(alphas,lambda, tau, par);

space_dim=3;

Epsilon_norm=[rgamma*ones(3,1),rfloquet*norm(intval(v),1)*ones(3,1)+r_v*sum(normQ,2)+3*rfloquet*r_v*ones(3,1)];
eps_1=max(sup(Epsilon_norm(:,2)));
alphas(1).eps=Epsilon_norm(:,1);
alphas(2).eps=Epsilon_norm(:,2);
par2=[m_manifold,space_dim, dim_man,N_manifold];
disp(['Validation of the High-order coefficients, m_manifold = ' num2str(m_manifold) ', N_manifold = ' num2str(N_manifold)]); 
tic
[rad_param,alphas]=proof_high_order_terms_lorenz(f_int,Jac, Jac_int,midrad(lambda,r_v),tau,par,par2,nu,alphas,Epsilon_norm);
toc
eps_al=rad_param;
for i=2:N_manifold
    alphas(i+1).eps=ones(3,1)*rad_param;
end

%%%%%
if N>12
   m_manifold2=100;
   par2(1)=m_manifold2;
 
       
   [alphas,aquad13,aquad12]=proof_super_term_lorentz(intval(nu),tau,alphas,lambda,N,par,par2);
end


display(['rho' ' nu_gamma  ' ' nu  '   '  tau' ])
display([num2str(rho) '   ' num2str(nu_gamma) '   ' num2str(nu) '   ' num2str(tau)])
 
display(['m_gamma ' ' m_Q '  ' m_manifold'    ' N_manifold '  'lambda' ])
display([num2str(m_gamma) '       ' num2str(mQ) '      ' num2str(m_manifold) '       ' num2str(N_manifold) '  ' num2str(lambda)])

display([' r_gamma   ' '  r_floquet  '  '   r_eigs   '     '   r_paramet.'  ])
display([num2str(rgamma) '   ' num2str(rfloquet) '   ' num2str(r_v) '   ' num2str(rad_param)])




%%%%%% COMPUTE VALIDATION VALUES %%%%%
ipi=intval('pi');
i3=intval(3);
i2=intval(2);
i1=intval(1);
itau=midrad(tau,rgamma);
ieps_1=intval(eps_1);
ieps_al=intval(eps_al);
Gamma=gamma2T;
m=(size(Gamma,2)+1)/2;
iT=i2*itau;
T=mid(iT);
tol=10^-11;


r=inf(iT/(intval(2)*intval(pi))*log(intval(nu)));
ir=intval(r);
ilambda=midrad(lambda,r_v);

%%% choice of NU %%%%

iNU=intval(NU);
ErrorBound=max(sup(aposteriori_error_bound(alphas,iNU)));


%%%% compute A %%%%
Df_gamma_r=sum(abs(intval(Ak_tau_per)).*repmat(reshape(exp(i2*ipi*ir/itau).^(intval(abs(-m_gamma+1:m_gamma-1))),1,1,[]),3,3),3);
A=norm(sup(Df_gamma_r+[0,0,0;rgamma,0,rgamma;rgamma,rgamma,0]),inf); % kappa


%%%%% Compute Ctilde %%%%
   %%%%%% r-norm of alphas and norm of Atilde $$$$
a_r=intval(zeros(3,N_manifold));
   %% alpha=1%%%
a_r(:,1)=sum(abs(alphas(2).fc).*repmat(exp(i2*ipi*ir/iT).^(intval(abs(-m_manifold+1:m_manifold-1))),3,1),2)+Epsilon_norm(:,2);
   %% alpha>1%%%
for al=2:N_manifold
    a_r(:,al)=sum(abs(alphas(al+1).fc).*repmat(exp(i2*ipi*ir/iT).^(intval(abs(-m_manifold+1:m_manifold-1))),3,1),2)+intval(alphas(al+1).eps);
end
for al=N_manifold+1:N
    a_r(:,al)=intval(alphas(al+1).eps);
end

Atilde_r_infty=intval(zeros(1,N));
for al=1:N
   Atilde_r_infty(al)=norm([a_r(3,al),0,a_r(1,al);a_r(2,al),a_r(1,al),0],inf);
end

Ctilde=exp(sum(Atilde_r_infty.*(iNU.^intval([1:1:N]))./abs(ilambda*intval([1:1:N]))));

%%%%%%% compute rho' %%%%
%rho1=sum(max(sup(a_r),[],1).*(NU.^(1:1:N_manifold)));
%%%%%%  define rho
%Rho=1.1*rho1;
%%%%%% compute M1,M2, mustar
mu_star=sup(abs(real(ilambda)));
M1=2;
M2=1;


%%%%%    A-posteriori validation %%%%
val1=0;
val2=0;
if (N+1)>A/mu_star
    val1=1;
else
    'error1'
end

delta=sup((i2*Ctilde)*intval(ErrorBound)/((intval(N+1))*intval(mu_star)-intval(A)));
if delta<inf(exp(-i1)*(intval((N+1))*intval(mu_star)-intval(A))/(i2*i3*ipi*intval(M1)*intval(M2)*Ctilde))%(2*Ctilde)*ErrorBound/((N_manifold+1)*mu_star-A)<delta
    val2=1;
else
    'error2'
end

if val1*val2>0
    disp('The parameterization is rigorously validated')
    display(['r = ' num2str(r)]);
    display(['NU = ' num2str(NU)]);
    display(['r-NU-norm of H = ' num2str(delta)]);
    display(['A-posteriori Error bound = ' num2str(ErrorBound)]);
else
    display('validation fails');
end

plot_parametrization_complex(T,alphas,NU);
matlabpool('close')
