

function [f,f_int,Jac, Jac_int, alphas,vec_al]=compute_fN_manifold_lorenz(alphas,lambda, T, par)

N=size(alphas,2)-1;
m=(size(alphas(1).fc,2)+1)/2;

vec_al=[];
for al=2:N
   a_al=(alphas(al+1).fc).';
   vec_al=[vec_al;reshape(a_al,[],1)];
end

f=compute_map(alphas,lambda, T,par);
disp(['Inf-norm of F=', num2str(norm(f,inf))])
while norm(f,inf)>10^-14
Jac=compute_gradient(alphas,lambda, T, par);

vec_al=vec_al-Jac\f;

% reconstruct alphas
for n=2:N
    al_N=vec_al((n-2)*3*(2*m-1)+1:(n-1)*3*(2*m-1));
    al_N=reshape(al_N,[],3);
    alphas(n+1).fc=al_N.';
end

f=compute_map(alphas,lambda, T, par);
disp(['Inf-norm of F=', num2str(norm(f,inf))])
end

f=compute_map(alphas,lambda, T, par);
f_int=compute_map_int(alphas,lambda, T, par);
Jac=compute_gradient(alphas,lambda, T, par);
Jac_int=compute_gradient_int(alphas,lambda, T, par);

disp('Jac is the approximate Jacobian');
disp('Jac_int is the intval jacobian');
end

%%%%%%%%%%%%%%%%
    function fN=compute_map(alphas,lambda, T,par)
rho=par(2);
sigma=par(1);
beta=par(3);
N=size(alphas,2)-1;
m=(size(alphas(1).fc,2)+1)/2;

        fN=[];

for al=2:N
   a1a3=compute_quad(1,3,al,alphas);
   a1a2=compute_quad(1,2,al,alphas);

  a_al=(alphas(al+1).fc).';
    f_ak=(pi*1i/(T)*repmat([-m+1:1:m-1].',1,3)+lambda*al).*a_al-[-sigma*(a_al(:,1)-a_al(:,2)),rho*a_al(:,1)-a_al(:,2)-a1a3,-beta*a_al(:,3)+a1a2];
                                                                
    fN=[fN;reshape(f_ak,[],1)];

end
    end

    %%%%%%%%%%%%%%%%%%%%%%
    
function Jacobian=compute_gradient(alphas,lambda, T, par)
rho=par(2);
sigma=par(1);
beta=par(3);

N=size(alphas,2)-1;
m=(size(alphas(1).fc,2)+1)/2;

Id=eye(2*m-1);
Jacobian=zeros(3*(N-1)*(2*m-1));
for a=2:N
    for b=2:N
    Jab=zeros(3*(2*m-1));
    for i=1:3
        for j=1:3
            if i==j && a==b
             Jabij=2*pi/(2*T)*1i*diag(-m+1:1:m-1)+lambda*a*Id;
            else              
            Jabij=zeros(2*m-1);
            end
             
               if i==1 
                   
                    if j==1 
                       Jabij=Jabij+(a==b)*sigma*Id; 
                    elseif j==2
                       Jabij=Jabij-(a==b)*sigma*Id;
                        
                    end
               elseif i==2
                  
                    if j==1
                        Jabij=Jabij-(a==b)*rho*Id+tildeA(3,a-b,alphas);
                    elseif j==2
                        Jabij=Jabij+(a==b)*Id;
                    else 
                        Jabij=Jabij+tildeA(1,a-b,alphas);
                    end
                else %i=3%
                 
                    if j==1
                        Jabij=Jabij-tildeA(2,a-b,alphas);
                    elseif j==2
                        Jabij=Jabij-tildeA(1,a-b,alphas);
                    else 
                        Jabij=Jabij+(a==b)*beta*Id;
                    end
            end
           Jab((i-1)*(2*m-1)+1:i*(2*m-1),(j-1)*(2*m-1)+1:j*(2*m-1))=Jabij;
        end
    end
    
    Jacobian((a-2)*(3*(2*m-1))+1:(a-1)*3*(2*m-1),(b-2)*(3*(2*m-1))+1:(b-1)*3*(2*m-1))=Jab;
        
    end
end




end

%%%%%%%%%%%%%
function fN=compute_map_int(alphas,lambda, T,par)


ipi=intval('pi');
i1i=intval(1i);

iT=intval(T);
ilambda=intval(lambda);
rho=intval(par(2));
sigma=intval(par(1));
beta=intval(8)/intval(3);

N=size(alphas,2)-1;
m=(size(alphas(1).fc,2)+1)/2;

        fN=[];

for al=2:N
   a1a3=compute_quad_int(1,3,al,alphas);
   a1a2=compute_quad_int(1,2,al,alphas);

  a_al=intval((alphas(al+1).fc).');
  a_al=[intval(zeros(m-1,3));a_al;intval(zeros(m-1,3))];
    f_ak=(ipi*i1i/(iT)*repmat(intval([-2*m+2:1:2*m-2].'),1,3)+ilambda*intval(al)).*a_al-[-sigma*(a_al(:,1)-a_al(:,2)),rho*a_al(:,1)-a_al(:,2)-a1a3,-beta*a_al(:,3)+a1a2];
                                                                
    fN=[fN;reshape(f_ak,[],1)];

end
end
    
%%%%%%%%%%%%%%%

function Jacobian=compute_gradient_int(alphas,lambda, T, par)

i2=intval(2);
ipi=intval('pi');
i1i=intval(1i);

iT=intval(T);
ilambda=intval(lambda);

rho=intval(par(2));
sigma=intval(par(1));
beta=intval(8)/intval(3);

N=size(alphas,2)-1;
m=(size(alphas(1).fc,2)+1)/2;

Id=intval(eye(2*m-1));
Jacobian=intval(zeros(3*(N-1)*(2*m-1)));
for a=2:N
    ia=intval(a);
    for b=2:N
    Jab=intval(zeros(3*(2*m-1)));
    for i=1:3
        for j=1:3
            if i==j && a==b
             Jabij=i2*ipi/(i2*iT)*i1i*intval(diag(-m+1:1:m-1))+ilambda*ia*Id;
            else              
            Jabij=intval(zeros(2*m-1));
            end
             
               if i==1 
                
                    if j==1 
                       Jabij=Jabij+(a==b)*sigma*Id; 
                    elseif j==2
                       Jabij=Jabij-(a==b)*sigma*Id;
                        
                    end
               elseif i==2
                  
                    if j==1
                        Jabij=Jabij-(a==b)*rho*Id+intval(tildeA(3,a-b,alphas));
                    elseif j==2
                        Jabij=Jabij+(a==b)*Id;
                    else 
                        Jabij=Jabij+intval(tildeA(1,a-b,alphas));
                    end
                else %i=3%
                 
                    if j==1
                        Jabij=Jabij-intval(tildeA(2,a-b,alphas));
                    elseif j==2
                        Jabij=Jabij-intval(tildeA(1,a-b,alphas));
                    else 
                        Jabij=Jabij+(a==b)*beta*Id;
                    end
            end
           Jab((i-1)*(2*m-1)+1:i*(2*m-1),(j-1)*(2*m-1)+1:j*(2*m-1))=Jabij;
        end
    end
    
    Jacobian((a-2)*(3*(2*m-1))+1:(a-1)*3*(2*m-1),(b-2)*(3*(2*m-1))+1:(b-1)*3*(2*m-1))=Jab;
        
    end
end
end




function quad=compute_quad(j1,j2,n,alphas)
quad=zeros(length(alphas(1).fc),1);
for al1=0:n
    al2=n-al1;

    quad=quad+quadratic_sumFFT_complex(alphas(al1+1).fc(j1,:),alphas(al2+1).fc(j2,:));
end
end

%%%%%%%%%%%%
function quad=compute_quad_int(j1,j2,n,alphas)
m=(size(alphas(1).fc,2)+1)/2;

quad=intval(zeros(4*m-3,1));
for al1=0:n
    al2=n-al1;
    c1=alphas(al1+1).fc(j1,:); 
    c2=alphas(al2+1).fc(j2,:);
    c1=[intval(zeros(1,m-1)),c1,intval(zeros(1,m-1))]; % in order to have all the terms in the convolution
    c2=[intval(zeros(1,m-1)),c2,intval(zeros(1,m-1))];
    quad=quad+quadratic_sumFFT_complex_int(c1,c2);
end
end

function [s]=quadratic_sumFFT_complex(c1,c2)
if size(c1,2)>1
    c1=c1.';
end
if size(c2,2)>1
    c2=c2.';
end
m=(length(c1)+1)/2;

ta1=[zeros(m,1);c1;zeros(m,1)];
tu1=ifft(ifftshift(ta1));

ta2=[zeros(m,1);c2;zeros(m,1)];
tu2=ifft(ifftshift(ta2));

F=fftshift(fft(tu1.*tu2));

s=(4*m-1)*F(m+1:3*m-1);

end

function tildeA=tildeA(k,c,alphas)
if c<0
    tildeA=0;
    return
end
a=alphas(c+1).fc(k,:);
m=(length(a)+1)/2;

a=[zeros(1,m-1),a,zeros(1,m-1)];
center=2*m-1;
tildeA=zeros(2*m-1);
for N=-m+1:m-1
   tildeA(m+N,:)=a(center+N-[-m+1:1:m-1]); 
end
end


function [s]=quadratic_sumFFT_complex_int(a1,a2)


i2=intval('2');
i4=intval('4');

if size(a1,2)>1
    a1=a1.';
end
if size(a2,2)>1
    a2=a2.';
end
M=(length(a1)+1)/2;



% We make sure that the inputs are powers of 2 %
M1=(2^ceil(log(4*M-1)/log(2))-4*M)/2;   % Hence 4*M+2*M1 is a power of 2 %
ta1=[intval(zeros(M+M1+1,1));a1;intval(zeros(M+M1,1))];
ta2=[intval(zeros(M+M1+1,1));a2;intval(zeros(M+M1,1))];
tu1=verifyfft(ifftshift(ta1),-1);
tu2=verifyfft(ifftshift(ta2),-1);
F=fftshift(verifyfft(tu1.*tu2,1));
s=(i4*intval(M)+i2*intval(M1))*F(M1+M+2:M1+3*M,1);
end
