function J=jacobian_floquet_lorenz_nosym(R,Q,A,T)

% Jacobian matrix for the Floquet system in case of the Lorenz equation 

%global   T 


m=(size(Q,3)+1)/2;  % Q_k, |k|<m
MA=(size(A,3)+1)/2;
dim=size(A,1);
if MA<2*m
    A=cat(3,zeros(3,3,2*m-MA),A,zeros(3,3,2*m-MA));
    MA=(size(A,3)+1)/2;
end

Ablock=zeros(dim^2,dim^2,size(A,3));

Qblock=zeros(dim^2,dim^2,size(Q,3));



for k=1:size(A,3)
Ablock(:,:,k)=kron(eye(dim),A(:,:,k));
end


for k=1:size(Q,3)
  Qblock(:,:,k)=kron(eye(dim),Q(:,:,k));  
end


Rstar=kron(R.',eye(3));


indexj=-(m-1):1:(m-1);

jac_phase=[repmat(eye(9),1,m-1),zeros(9),eye(9),repmat(eye(9),1,m-1)];

jac_0=-Ablock(:,:,MA+0-indexj);
jac_0(:,:,m+0)=jac_0(:,:,m+0)+Rstar;
jac_0=cat(3,jac_0(:,:,1:1:m-1),Qblock(:,:,m+0),jac_0(:,:,m:1:end));

jac_0=reshape(jac_0,dim^2,[]);


J_zero=[jac_phase;jac_0];


J=zeros(dim^2*2*m);
for k=-(m-1):-1
   jac_k=-Ablock(:,:,MA+k-indexj);
   jac_k(:,:,m+k)=jac_k(:,:,m+k)+(1i*k*pi/T*eye(dim^2)+Rstar);
   jac_k=cat(3,jac_k(:,:,1:1:m-1),Qblock(:,:,m+k),jac_k(:,:,m:1:end));
   jac_k=reshape(jac_k,dim^2,[]);
   J((m+k-1)*dim^2+1:(m+k)*dim^2,:)=jac_k;
end
J((m-1)*dim^2+1:(m+1)*dim^2,:)=J_zero;
for k=1:(m-1)
   jac_k=-Ablock(:,:,MA+k-indexj);
   jac_k(:,:,m+k)=jac_k(:,:,m+k)+(1i*k*pi/T*eye(dim^2)+Rstar);
   jac_k=cat(3,jac_k(:,:,1:1:m-1),Qblock(:,:,m+k),jac_k(:,:,m:1:end));
   jac_k=reshape(jac_k,dim^2,[]);
   J((m+k)*dim^2+1:(m+k+1)*dim^2,:)=jac_k;
end



end