function [R,Q,JJ]=newton_floquet_lorenz(gamma,Ak,m,par,T)

%Newton scheme for the solution of the Floquet system, in unknowns R, C_k
%The initial data for R is given by integration of the variational system
%up to 2*Period and then by computing the matrix logarithm.
%The initial point for C_k is obtained by integrating the Floquet system
%using the above R and then by extracting the Fuorier coefficients 


sigma=par(1);
rho=par(2);
beta=par(3);

%global rho sigma beta gamma_0 gamma_k T

options=odeset('AbsTol',1e-16,'RelTol',1e-11);
gamma_0=gamma(:,1);
gamma_k=gamma(:,2:end);
sol_ini=real(gamma_0);
for k=1:size(gamma_k,2)
    sol_ini=sol_ini+2*real(gamma_k(:,k));
end
  
    
M=lor_monodromy2T;                   % first guess for the monodromy matrix
R=logm(real(M))/(2*T);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% integration of f'=Jac(gamma(t))*f-f*R, f(0)=I
inispace=sol_ini;
inif=eye(3);
ini=[inispace;reshape(inif,9,1)];

[t,y]=ode45(@system,[0,2*T],ini,options);       
[t,y]=ode45(@system,linspace(0,2*T,size(y,1)),ini,options);
N=length(t);

for j=1:9                                                                  % fourier coefficients of the solution f(t)
    Comp(:,j)=fft(y(:,3+j))/N;
end

for j=1:m
    Q(:,:,j)=reshape(Comp(j,:),3,3);
end
clear Comp

%%%%%%%%%% Newton scheme for R and C


Q=cat(3,flipdim(conj(Q(:,:,2:end)),3),Q);
m=(size(Q,3)+1)/2;
Ak=cat(3,zeros(3,3,2*m),Ak,zeros(3,3,2*m));



NewtonQR
while norm(Q(:,:,end)+Q(:,:,end-1))>10^-16
    Q=cat(3,zeros(3,3,10),Q,zeros(3,3,10));
    Ak=cat(3,zeros(3,3,10),Ak,zeros(3,3,10));
    m=m+10;
    NewtonQR;
    display(['norm(Q(:,:,end)+Q(:,:,end-1)) = ',num2str(   norm(Q(:,:,end)+Q(:,:,end-1)))])
    
  
end
    function NewtonQR
    Ntest=0;
    F=map_floquet(Ak,Q,R,T);
    X=reshape(cat(3,Q(:,:,1:m-1),R,Q(:,:,m:end)),[],1);
    JJ=  jacobian_floquet_lorenz_nosym(R,Q,Ak,T);
while norm(F,inf)>9*1e-14  


X1=X-JJ\F;
Q=cat(3,reshape(X1(1:(m-1)*9),3,3,m-1),reshape(X1(9*m+1:end),3,3,m));
R=reshape(X1((m-1)*9+1:m*9),3,3,1);
F=map_floquet(Ak,Q,R,T);
display(['Inf-norm of F(x) = ',num2str( norm(F,inf))])
Ntest=Ntest+1;
    if Ntest>9
        if norm(F,inf)<5*1e-13
            break
        elseif norm(F,inf)>1
            disp('Newton scheme for the Floquet system does not converge');
            disp('Try to change  m');
            R=[];
            Q=[];
            return
        else
            Ntest=Ntest-2;
        end
    end
  X=reshape(cat(3,Q(:,:,1:m-1),R,Q(:,:,m:end)),[],1);   
  JJ=jacobian_floquet_lorenz_nosym(R,Q,Ak,T);
end
R=real(R);

end
%%%%%%%%%%%%%%%
    function Mon=lor_monodromy2T
                        
        X=eye(3);
        ini=[sol_ini;reshape(X,[],1)];
        [t,y]=ode45(@systemMonodromy,[0 2*T],ini,options);   % integrate linearized system   
        Mon=reshape(y(end,4:end),3,3);
        
        
        function df=systemMonodromy(t,x)
            
            Mon=reshape(x(4:end),3,3);            
            J=[-sigma, sigma, 0;...
                rho-x(3), -1, -x(1);...
                x(2), x(1),-beta];
                        
            df(1,1)=sigma*(x(2)-x(1));
            df(2,1)=rho*x(1)-x(2)-x(1)*x(3);
            df(3,1)=x(1)*x(2)-beta*x(3);
            dfM=J*Mon;
            df=[df;reshape(dfM,9,1)];
            
        end
        
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function df=system(t,x)
        
        f=reshape(x(4:end),3,3);
        
        J=[-sigma, sigma, 0;...
            rho-x(3), -1, -x(1);...
            x(2), x(1),-beta];
        
        df(1,1)=sigma*(x(2)-x(1));
        df(2,1)=rho*x(1)-x(2)-x(1)*x(3);
        df(3,1)=x(1)*x(2)-beta*x(3);
        dff=J*f-f*R;
        df=[df;reshape(dff,9,1)];
    end
end
%%%%%%
function F=map_floquet(Ak,Q,R,T)
        m=(size(Q,3)+1)/2;
        dim=size(R,1);
        F=zeros(dim,dim,2*(m-1)+1+1);
        for k=-(m-1):-1
           F(:,:,m+k)=1i*k*pi/T*Q(:,:,m+k)+Q(:,:,m+k)*R-convAQ(k,Ak,Q) ;
        end
          F(:,:,m)=sum(Q,3)-eye(dim);
          for k=0:m-1
               F(:,:,m+1+k)=1i*k*pi/T*Q(:,:,m+k)+Q(:,:,m+k)*R-convAQ(k,Ak,Q) ;
          end
          
          F=reshape(F,[],1);
end
    function Conv=convAQ(k,A,Q)
        dimA=size(A,1);
        medA=(size(A,3)+1)/2;
        m=(size(Q,3)+1)/2;
    
        l=-(m-1):1:(m-1);
        j=-l+k;
        Conv=reshape(A(:,:,medA+j),dimA,[])*reshape(permute(Q(:,:,m+l),[1,3,2]),[],dimA);
        
        
    end
