function [rfloquet,Q_norm]=radii_pol_floquet_lorenz_analyt_comp(Ak,r_A,R,Q,T,Jm,nu)

iT=intval(T);
iR=intval(R);
iQ=intval(Q);
i1=intval(1);
i2=intval(2);
ipi=intval('pi');
inu=intval(nu);
ir_A=intval(r_A);

dim=size(R,1);
idim=intval(dim);
m=(size(Q,3)+1)/2;
MA=(size(Ak,3)+1)/2;
Q_cell=mat2cell(reshape(Q,dim,[]),dim,dim*ones(1,size(Q,3)));
%%%%%%%%%%%
J=[2,3,6,8];
Jhat=[2,3,5,6,8,9];
%%%%%%%%%%%%
Nu_m=repmat(inu.^(reshape(abs(intval([-(m-1):1:(m-1)])),1,1,[])),dim^2,dim^2);

inv_Nu_m=repmat(inu.^(-reshape(abs(intval([-(m-1):1:(m-1)])),1,1,[])),dim^2,dim^2);
% Q(i,j) nu norm
Q_norm=sum( abs(intval(Q)).*Nu_m(1:dim,1:dim,:),3);
% define Rtilde Qtilde and Qhat
Rtilde=(kron(R.',eye(dim)));
iRtilde=intval(Rtilde);

Qtilde=zeros(dim^2,dim^2,2*m-1);
Qhat=Qtilde;
for j=1:size(Q,3)
 Qtilde(:,:,j)=(kron(Q(:,:,j).',eye(dim)));%mat2cell(reshape(Q,dim,[]),dim,dim*ones(1,size(Q,3)));
 Qhat(:,:,j)=(kron(eye(dim),Q(:,:,j)));
end
Qtilde_cell=mat2cell(reshape(Qtilde,dim^2,[]),dim^2,dim^2*ones(1,size(Qtilde,3)));

% define Ahat
Ahat=zeros(dim^2,dim^2,2*MA-1);
for j=1:size(Ak,3)
    Ahat(:,:,j)=kron(eye(dim),Ak(:,:,j));
end
iAhat=intval(Ahat);
Ahat_cell=mat2cell(reshape(Ahat,dim^2,[]),dim^2,dim^2*ones(1,size(Ahat,3)));


% force Jm to preserve the symmetry
J11=Jm(1:(m-1)*dim^2,1:(m-1)*dim^2);
J12=Jm(1:(m-1)*dim^2,(m+1)*dim^2+1:end);

C11=mat2cell(conj(J11),dim^2*ones(1,m-1),dim^2*ones(1,m-1));
C12=mat2cell(conj(J12),dim^2*ones(1,m-1),dim^2*ones(1,m-1));

D22=flip(flip(C11,1),2);
D21=flip(flip(C12,1),2);

C22=cell2mat(D22);
C21=cell2mat(D21);
Jmnew=Jm;
Jmnew((m+1)*dim^2+1:end,1:(m-1)*dim^2)=C21;
Jmnew((m+1)*dim^2+1:end,(m+1)*dim^2+1:end)=C22;
Jm=Jmnew;

% compute lambda and Lambda_m
Lbd=Lambda;
mid_lambda=(size(Lbd,2)+1)/2;
invLm=inv(Lbd{mid_lambda+m});

Lambda_m=inf(i1/norm(invLm,inf));
iLambda_m=intval(Lambda_m);

INVLambda=cellfun(@(x) inv(x), Lbd,'UniformOutput',false);  % JKKtot

%%%%%%%% arrange the matrix Jm %%%%%

J_cell=mat2cell(Jm,dim^2*ones(1,2*m),dim^2*ones(1,2*m));  % entire Jm into cells
J_cellK=J_cell;
J_cellK(:,m)=[];
J_cellK(m,:)=[];                                         % (Jm)(k,l) ,not star, into cell
J_col_star_cell=[J_cell(1:m-1,m);J_cell(m+1:end,m)];     % star_column of Jm
J_row_star_cell=[J_cell(m,1:m-1),J_cell(m,m+1:end)];     % star_row of Jm
iJm=intval(Jm);


%%%%%% Norms of J ( componentwise) %%%%
% nu-norm of the columns of J %%%%
J_col_star_norm_nu=sum(intval(abs(cat(3,J_col_star_cell{:}))).*Nu_m,3);

J_cols_norm_nu=intval(zeros(dim^2,dim^2,2*m-1));
for k=-(m-1):m-1
    J_cols_norm_nu(:,:,m+k)=sum(abs(cat(3,J_cellK{:,m+k})).*Nu_m,3);
end
% norm of Jm
Jm_norm=max(sup(J_cols_norm_nu.*inv_Nu_m),[],3);

% norm of J 
J_norm=max(cat(3,Jm_norm,sup(i1/iLambda_m*ones(dim^2))),[],3);


% dual inf-nu norms
J_row_star_inf_norm=max(sup(intval(abs(cat(3,J_row_star_cell{:}))).*inv_Nu_m),[],3);

J_rows_inf_norm=zeros(dim^2,dim^2,2*m-1);
for k=-(m-1):(m-1)
   J_rows_inf_norm(:,:,m+k)=max(sup(abs(cat(3,J_cellK{m+k,:})).*inv_Nu_m),[],3);
end



% convolution A*Q
conv_cell=define_convolution(Ak,Q);
mid_conv=(size(conv_cell,2)+1)/2;

% compute F
f_star=sum(iQ,3)-intval(eye(dim));
fk=cellfun(@(x,y,z) 1i*ipi/iT*intval(x)*intval(y) + intval(y)*iR -z,num2cell(-(m-1):m-1), Q_cell,conv_cell(mid_conv+[-(m-1):(m-1)]),'UniformOutput', false) ;
F=[];
for j=1:m-1
    F=[F;reshape(fk{j},[],1)];
end
F=[F;reshape(f_star,[],1)];
for j=m:length(fk)
    F=[F;reshape(fk{j},[],1)];
end

% compute jacobian at the numerical solution 

Jac=jacobian_floquet(iRtilde,Qhat,Ahat,iT);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%  Z0 %%%%%%%%

B=sup(abs(intval(eye(size(Jm)))-iJm*Jac));
B_cell=mat2cell(B,dim^2*ones(1,2*m),dim^2*ones(1,2*m));
Bstar_star=B_cell{m,m};
B_cellK=B_cell;
B_cellK(:,m)=[];
B_cellK(m,:)=[];   

B_row_star=cat(3,B_cell{m,1:m-1},B_cell{m,m+1:end}); 
B_col_star=cat(3,B_cell{1:m-1,m},B_cell{m+1:end,m});

%%%%% norms of B %%%%%
B_col_star_norm_nu=sum(intval(B_col_star).*Nu_m,3);
B_row_star_inf_norm=max(sup(intval(B_row_star).*inv_Nu_m),[],3);   % dual norm of star_row of B

B_cols_norm_nu=intval(zeros(dim^2,dim^2,2*m-1));
for k=-(m-1):m-1
    B_cols_norm_nu(:,:,m+k)=sum(abs(intval(cat(3,B_cellK{:,m+k}))).*Nu_m,3);
end

% norm of B 
B_norm=max(sup(B_cols_norm_nu.*inv_Nu_m),[],3);

%%%% Z0^star %%%
Z0_star=sum(intval(Bstar_star),2)+sum(intval(B_row_star_inf_norm),2);
Z0_star=Z0_star+ir_A*i2*sum(intval(J_row_star_inf_norm(:,Jhat)),2);
%%% Z0^nu %%%
Z0_nu=sum(B_col_star_norm_nu,2)+sum(intval(B_norm),2)+i2*ir_A*sum(intval(Jm_norm(:,Jhat)),2);

display(['max of Z0 = ',num2str( norm([Z0_star.sup;Z0_nu.sup],inf))])

 %%%%%  BOUND Z1 %%%%%
 %% Z1 star %%%
 
 %%% compute the operator M and its dual norm %%%%%
 %%%%%%   computation of the matrices M_k used in the definition of Y^star
Ahatini=[flip(Ahat_cell,2),mat2cell(zeros(dim^2,dim^2*(2*m-2)),dim^2,dim^2*ones(1,2*m-2))];
AhatMat_cell=cell(2*m-1,size(Ahatini,2));

AhatMat_cell(1,:)=Ahatini;
    for i=1:2*m-2
        Ahatini=circshift(Ahatini,[1,1]);
        AhatMat_cell(i+1,:)=Ahatini;
    end    

midb=(size(AhatMat_cell,2)+1)/2;
AhatMat_cell(:,midb+[-(m-1):(m-1)])=[];

J_minus_star_col=J_cell;
J_minus_star_col(:,m)=[];
JA=intval(repmat(cell2mat(J_cell(:,m)),1,size(AhatMat_cell,2)))-intval(cell2mat(J_minus_star_col))*intval(cell2mat(AhatMat_cell));

MK=reshape(JA((m-1)*dim^2+[1:dim^2],:),dim^2,dim^2,[]);

k1_vect=[-MA-m+2:-m,m:MA+m-2];
k1(1,1,1:length(k1_vect))=abs(k1_vect);
inv_Nu_k1=repmat(inu.^(-intval(k1)),dim^2,dim^2);
M_norm_inf=max(sup(abs(MK).*inv_Nu_k1),[],3);
M_norm_inf=max(M_norm_inf, sup(i1/(inu^(intval(m+MA-1)))*abs(intval(J_col_star_cell{m}))));


Z1_star=sum(intval(M_norm_inf),2)+i2*ir_A*sum(intval(J_row_star_inf_norm(:,Jhat)),2);
 
%%%%% Z1_nu %%%%%%
% compiute the norm of the operator N
% first piece |b|<m
indexm=[-2*m-MA:-m,m:2*m+MA];
Q1=intval(zeros(dim^2,dim^2,2*m-1));
Ahatj=intval(max(abs(Ahat),[],1));
b=-(m-1);
while b<=(m-1)
    indexk=indexm(abs(indexm-b)<MA);
    if isempty(indexk)==1
        b=-b+1;
        indexk=indexm(abs(indexm-b)<MA);
    end
    k1=intval(repmat(reshape(abs(indexk)-abs(b),1,1,[]),1,dim^2));
    Q1(:,:,m+b)=repmat((i1/iLambda_m*sum(Ahatj(:,:,MA+indexk-b).*inu.^(k1),3)),dim^2,1);
    b=b+1;
end

% second piece  m<= |b|<=m+MA-2
JA((m-1)*dim^2+[1:dim^2],:)=[];


%%%%%%%
KK=[-(2*m+MA-3):1:-m,m:1:2*m+MA-3];
cont=1;
Q2=intval(zeros(dim^2,dim^2,2*(MA-1)));
InvLbd_inf=cell2mat(cellfun(@(x) sup(norm(x,inf)), INVLambda,'UniformOutput',false));
for b=-(m+MA-2):-m
    indexk=KK(abs(KK-b)<MA & KK~=b);
    k1=intval(repmat(reshape(abs(indexk)-abs(b),1,1,[]),1,dim^2));
    Ahatjnuk=Ahatj(:,:,MA+indexk-b).*inu.^(k1);
   Q2(:,:,cont)=Q2(:,:,cont)+repmat(sum(repmat(reshape(InvLbd_inf(mid_lambda+indexk),1,1,[]),1,dim^2).*Ahatjnuk,3),dim^2,1);   
   Q2(:,:,end-cont+1)=Q2(:,:,end-cont+1)+repmat(sum(repmat(reshape(InvLbd_inf(mid_lambda-indexk),1,1,[]),1,dim^2).*Ahatjnuk,3),dim^2,1);
   JAb=permute(reshape(permute(JA(:,(cont-1)*dim^2+[1:dim^2]),[2,1]),dim^2,dim^2,[]),[2,1,3]);
   JAb2= permute(reshape(permute(JA(:,(2*(MA-1)-cont)*dim^2+[1:dim^2]),[2,1]),dim^2,dim^2,[]),[2,1,3]);
 
   Q2(:,:,cont)=Q2(:,:,cont)+i1/(inu^(abs(intval(b))))*sum(abs(JAb).*Nu_m,3);
   Q2(:,:,end-cont+1)=Q2(:,:,end-cont+1)+i1/(inu^(abs(intval(b))))*sum(abs(JAb2).*Nu_m,3);
    
   cont=cont+1;
end

%third piece : m+MA-1<|b|<=2*(m+MA)
KK=[-(2*m+2*MA):1:-m,m:1:2*m+2*MA];
Q4=intval(zeros(dim^2,dim^2,2*(m+2)));
cont=1;
for b=-(2*m+MA):-m-MA+1
   indexk=KK(abs(KK-b)<MA & KK~=b);
   k1=intval(repmat(reshape(abs(indexk)-abs(b),1,1,[]),1,dim^2));
   Ahatjnuk=Ahatj(:,:,MA+indexk-b).*inu.^(k1);
   Q4(:,:,cont)=Q4(:,:,cont)+repmat(i1/Lambda_m*sum(Ahatjnuk,3),dim^2,1);   
   Q4(:,:,end-cont+1)=Q4(:,:,end-cont+1)+repmat(i1/Lambda_m*sum(Ahatjnuk,3),dim^2,1);
  
   Q4(:,:,cont)=Q4(:,:,cont)+i1/(inu^(abs(intval(b))))*J_col_star_norm_nu;%sum(abs(JAb).*Nu_m,3);
   Q4(:,:,end-cont+1)=Q4(:,:,end-cont+1)+i1/(inu^(abs(intval(b))))*J_col_star_norm_nu;%sum(abs(JAb2).*Nu_m,3);
    
   cont=cont+1;
end
% third piece : bound for |b|>=m+MA-1
Ahatj(:,:,MA)=0;


Q3=repmat(norm(inv(Lbd{mid_lambda+2*m}),inf)*sum(Ahatj.*repmat(inu.^intval(reshape(abs(-MA+1:1:MA-1),1,1,[])),1,dim^2),3),dim^2,1)+...
   +i1/(inu^(i2*intval(m)+intval(MA)))*J_col_star_norm_nu;

Qfin=cat(3,Q1.sup,Q2.sup,Q4.sup,Q3.sup);
norm_N=max(Qfin,[],3);
% error bound %%
Error=i2*ir_A*sum(intval(J_norm(:,Jhat)),2);
Z1_nu=sum(norm_N,2)+Error;
display(['max of Z1 = ',num2str( norm([Z1_star.sup;Z1_nu.sup],inf))])

if norm([Z1_star;Z1_nu],inf)>1
    'Z bound ecceeds 1, error'
    return
end

%%%%% BOUND Y %%%%%%%

% compute J*F
JtimesF=reshape(iJm*F,dim^2,[]);
JtimesFK=JtimesF;
JtimesFK(:,m)=[];

%%%%%%   computation of the matrices M_k used in the definition of Y^star
Qtildeini=[flip(Qtilde_cell,2),mat2cell(zeros(dim^2,dim^2*(2*m-2)),dim^2,dim^2*ones(1,2*m-2))];
QtildeMat_cell=cell(2*m-1,size(Qtildeini,2));
QtildeMat_cell(1,:)=Qtildeini;
    for i=1:2*m-2
        Qtildeini=circshift(Qtildeini,[1,1]);
        QtildeMat_cell(i+1,:)=Qtildeini;
    end    
QtildeMat=cell2mat(QtildeMat_cell);

MK=reshape((intval(cell2mat(J_row_star_cell))*intval(QtildeMat)),dim^2,dim^2,[]);
k2(1,1,1:2*(2*m-2)+1)=abs([-(2*m-2):1:2*m-2]);
inv_Nu_k2=repmat(inu.^(-intval(k2)),dim^2,dim^2);
M_norm_inf=max(sup(abs(MK).*inv_Nu_k2),[],3);

%%% Y^star %%%%
Ystar=abs(JtimesF(:,m))+ir_A*sum(intval(M_norm_inf(:,J)),2);


%%%% first part of Y^nu %%%%%%
JF_nu=sum(abs(JtimesFK).*repmat(inu.^(abs(intval(-(m-1):m-1))),dim^2,1),2);
for Absk=m:MA+m-2
    Factor_neg=Lbd{mid_lambda-Absk}\reshape(conv_cell{mid_conv-Absk},[],1);
    Factor_pos=Lbd{mid_lambda-Absk}\reshape(conv_cell{mid_conv-Absk},[],1);
    JF_nu=JF_nu+(abs(Factor_neg)+abs(Factor_pos))*inu^intval(Absk);
end

% second part of Y^nu %%%%
% Pkl is the matrix of matrices as in the paper where |k|<m
% compiute the firstsum and secondsum involved in the first max
% then compute the second max that is the bound for l2>2m-2
% hence compute the max between the 2 terms

K=[-(3*m-3): -m,m:(3*m-3)];

L=-(m-1):(m-1);
%%%%%%%%%%%%%%

P=zeros(dim^2,dim^2,2*(2*m-2)+1);
cont=1;
for l2=-(2*m-2):-1%(2*m-2)
    lindex=L(abs(L-l2)<m);
    first=permute(reshape(intval((cell2mat(J_cellK(:,m+lindex)))*intval(cat(1,Qtilde_cell{m+lindex-l2}))).',dim^2,dim^2,[]),[2,1,3]);
    
    first=sum(abs(first).*(Nu_m/(inu^(intval(abs(l2))))),3);
    kindex=K(abs(K-l2)<m);

        
    second_product_cell=cellfun(@(x,y) sup(abs(x*intval(y))), INVLambda(1,mid_lambda+kindex),Qtilde_cell(m+kindex-l2), 'UniformOutput',false);
    second=sum(cat(3,second_product_cell{:}).*repmat(inu.^(intval(reshape(abs(kindex)-abs(l2),1,1,[]))), dim^2,dim^2),3);
  
    P(:,:,cont)=sup(first+second);
    P(:,:,end-cont+1)=P(:,:,cont);
    cont=cont+1;
 
end
%%% l2=0 %%%%
  lindex=L;%(abs(L-l2)<m);
  first=permute(reshape(intval((cell2mat(J_cellK(:,m+lindex)))*intval(cat(1,Qtilde_cell{m+lindex}))).',dim^2,dim^2,[]),[2,1,3]);
  first=sum(abs(first),3);
  P(:,:,cont)=sup(first);
 
Max1=max(P,[],3);

%%%%%%%%%%%%%%%

norm_inf_j_Qtilde=max(sup(abs(intval(Qtilde))),[],1);

Max2=repmat(sum(intval(norm_inf_j_Qtilde).*(Nu_m(1,:,:)),3)/iLambda_m,dim^2,1);

normP=max(sup(Max1),sup(Max2));

JEF_nu=ir_A*sum(intval(normP(:,J)),2);
%%% Y^nu %%%
Ynu=sup(JF_nu)+JEF_nu;
%%%%%%%%%%%%%%%
display(['max of Y = ',num2str( norm([Ystar.sup;Ynu.sup],inf))])

%%%%% Z2 %%%%%%

Z2_star=i2*idim*sum(intval(J_row_star_inf_norm),2);
Z2_nu=i2*idim*sum(intval(J_norm),2);
display(['max of Z2 = ',num2str( norm([Z2_star.sup;Z2_nu.sup],inf))])
%%%%% radii polynomial
Y=[Ystar;Ynu];
Z0=[Z0_star;Z0_nu];
Z1=[Z1_star;Z1_nu];
Z2=[Z2_star;Z2_nu];
ip=[Z2,Z0+Z1-1,Y];
p=real(mid(ip));

I=zeros(size(p,1),2);
for k=1:size(p,1)
    R=roots(p(k,:));
    R=sort(R);
    Int=R';
    I(k,:)=Int;
end
I=[max(I(:,1)) min(I(:,2))]

if norm(imag(I))>0
    display('FAILURE !')
    I=[-1 1];
    success=0;
elseif I(1)<0
    display('FAILURE !')
    I=[-1 1];
    success=0;
else    
    r=1.1*I(1);%(I(1)+I(2))/2;
    ir=intval(r);
    pr=p(:,1)*ir^2+p(:,2)*ir+p(:,3);
    if max(sup(pr))<0
        display('SUCCESS !')
        success=1;
        rfloquet=r
        %display(['r = ',num2str(r)])
    else
        display('FAILURE ! max(sup(pr))>=0')
        success=0;
    end
end

%%%%%%%%%
 function L=Lambda
     L=cellfun(@(x) 1i*ipi/iT*intval(x)*intval(eye(dim^2))+iRtilde-iAhat(:,:,MA), num2cell(-(3*max(m,MA)-1):1:3*max(m,MA)-1),'UniformOutput',false);
  end

end



%%%%%
% 
function conv_cell=define_convolution(A,Q)
        m=(size(Q,3)+1)/2;  % Q_k, |k|<m
        MA=(size(A,3)+1)/2;
        dim=size(A,1);
        A=cat(3,zeros(dim,dim,MA+2*m-3-(MA-1)),A,zeros(dim,dim,MA+2*m-3-(MA-1)));
        A_cell=mat2cell(reshape(A,dim,[]),dim, dim*ones(1,size(A,3)));
        midA=(size(A_cell,2)+1)/2;
        Mat_cell=cell(2*(MA+m-2)+1,2*m-1);
        mid_Mat=(size(Mat_cell,1)+1)/2;
        Q_cell=mat2cell(reshape(Q,dim,[]),dim,dim*ones(1,size(Q,3)));
        
        k2=-(m-1):1:(m-1);
        for k=-(MA+m-2): MA+m-2
          Mat_cell(mid_Mat+k,:)=A_cell(midA+k-k2);   
        end
        Prod=intval(cell2mat(Mat_cell))*intval(cell2mat(Q_cell.'));
        conv_cell=mat2cell(Prod,dim*ones(1,size(Prod,1)/dim),3);
        conv_cell=conv_cell.';
    end
        

    
function J=jacobian_floquet(Rtilde,Qhat,Ahat,T)

ipi=intval('pi');
% Jacobian matrix for the Floquet system in case of the Lorenz equation 
%%%%%% matrix representation of the moltiplication A:M-> AM and Q:M->QM
dim=size(Rtilde,1);
MA=(size(Ahat,3)+1)/2;
m=(size(Qhat,3)+1)/2;  % Q_k, |k|<m

Ahat=cat(3,zeros(dim,dim,2*m-MA),Ahat,zeros(dim,dim,2*m-MA));
MA=(size(Ahat,3)+1)/2;

indexj=-(m-1):1:(m-1);
jac_phase=intval([repmat(eye(dim),1,m-1),zeros(dim),eye(dim),repmat(eye(dim),1,m-1)]);

Jac_0=-(Ahat(:,:,MA-indexj));
%Jac_0(:,:,m)=Jac_0(:,:,m)+Rtilde;

jac_0=intval(cat(3,Jac_0(:,:,1:m-1),Qhat(:,:,m),Jac_0(:,:,m:end)));
jac_0(:,:,m+1)=jac_0(:,:,m+1)+Rtilde;
jac_0=reshape(jac_0,dim,[]);
J_zero=[jac_phase;jac_0];

J=intval(zeros(dim*2*m));
count=0;
for k=-(m-1):m-1
    if k==0
        jac_k=J_zero;
        J(count*dim+[1:2*dim],:)=jac_k;
        count=count+2;
    else
   Jac_k=-intval(Ahat(:,:,MA+k-indexj));
   Jac_k(:,:,m+k)=Jac_k(:,:,m+k)+(1i*intval(k)*ipi/T*intval(eye(dim))+Rtilde);
   
   jac_k=intval(zeros(dim,dim,2*m));
   jac_k(:,:,1:m-1)=Jac_k(:,:,1:m-1);
   jac_k(:,:,m)=intval(Qhat(:,:,m+k));
   jac_k(:,:,m+1:end)=Jac_k(:,:,m:1:end);
     
   jac_k=reshape(jac_k,dim,[]);

   J(count*dim+[1:dim],:)=jac_k;
   count=count+1;
    end   
end

end


